@@ -10,6 +10,140 @@
 #' @export
 metadata <- list()
 
+#' Render R Markdown
+#'
+#' Render the input file to the specified output format using pandoc. If the
+#' input requires knitting then \code{\link[knitr:knit]{knit}} is called prior
+#' to pandoc.
+#'
+#' Note that the \pkg{knitr} \code{error} option is set to \code{FALSE} during
+#' rendering (which is different from the \pkg{knitr} default value of
+#' \code{TRUE}).
+#'
+#' For additional details on rendering R scripts see
+#' \link[=compile_notebook]{Compiling R scripts to a notebook}.
+#'
+#' If no \code{output_format} parameter is specified then the output format is
+#' read from the YAML front-matter of the input file. For example, the
+#' following YAML would yield a PDF document:
+#'
+#' \preformatted{
+#' output: pdf_document
+#' }
+#'
+#' Additional format options can also be specified in metadata. For example:
+#'
+#' \preformatted{
+#' output:
+#'   pdf_document:
+#'     toc: true
+#'     highlight: zenburn
+#' }
+#'
+#' Multiple formats can be specified in metadata. If no \code{output_format}
+#' is passed to \code{render} then the first one defined will be used:
+#'
+#' \preformatted{
+#' output:
+#'   pdf_document:
+#'     toc: true
+#'     highlight: zenburn
+#'   html_document:
+#'     toc: true
+#'     theme: united
+#' }
+#'
+#' Formats specified in metadata can be any one of the built in formats (e.g.
+#' \code{\link{html_document}}, \code{\link{pdf_document}}) or a format defined
+#' in another package (e.g. \code{pkg::custom_format}).
+#'
+#' If there is no format defined in the YAML then
+#' \code{\link{html_document}} will be used.
+#' @section R Markdown:
+#' R Markdown supports all of the base pandoc markdown features as well as some
+#' optional features for compatibility with GitHub Flavored Markdown (which
+#' previous versions of R Markdown were based on). See
+#' \code{\link{rmarkdown_format}} for details.
+#' @seealso
+#' \link[knitr:knit]{knit}, \link{output_format},
+#' \href{http://johnmacfarlane.net/pandoc}{pandoc}
+#' @param input The input file to be rendered. This can be an R script (.R),
+#' an R Markdown document (.Rmd), or a plain markdown document.
+#' @param output_format The R Markdown output format to convert to. The option
+#' \code{"all"} will render all formats defined within the file. The option can
+#' be the name of a format (e.g. \code{"html_document"}) and that will render
+#' the document to that single format. One can also use a vector of format
+#' names to render to multiple formats. Alternatively, you can pass an output
+#' format object (e.g. \code{html_document()}). If using \code{NULL} then the
+#' output format is the first one defined in the YAML frontmatter in the input
+#' file (this defaults to HTML if no format is specified there).
+#' @param output_file The name of the output file. If using \code{NULL} then the
+#' output filename will be based on filename for the input file. If a filename
+#' is provided, a path to the output file can also be provided. Note that the
+#' \code{output_dir} option allows for specifying the output file path as well,
+#' however, if also specifying the path, the directory must exist.
+#' @param output_dir The output directory for the rendered \code{output_file}.
+#' This allows for a choice of an alternate directory to which the output file
+#' should be written (the default output directory of that of the input file).
+#' If a path is provided with a filename in \code{output_file} the directory
+#' specified here will take precedence. Please note that any directory path
+#' provided will create any necessary directories if they do not exist.
+#' @param output_options List of output options that can override the options
+#' specified in metadata (e.g. could be used to force \code{self_contained} or
+#' \code{mathjax = "local"}). Note that this is only valid when the output
+#' format is read from metadata (i.e. not a custom format object passed to
+#' \code{output_format}).
+#' @param intermediates_dir Intermediate files directory. If a path is specified
+#' then intermediate files will be written to that path. If \code{NULL},
+#' intermediate files are written to the same directory as the input file.
+#' @param knit_root_dir The working directory in which to knit the document;
+#' uses knitr's \code{root.dir} knit option. If \code{NULL} then the behavior
+#' will follow the knitr default, which is to use the parent directory of the
+#' document.
+#' @param runtime The runtime target for rendering. The \code{static} option
+#' produces output intended for static files; \code{shiny} produces output
+#' suitable for use in a Shiny document (see \code{\link{run}}). The default,
+#' \code{auto}, allows the \code{runtime} target specified in the YAML metadata
+#' to take precedence, and renders for a \code{static} runtime target otherwise.
+#' @param clean Using \code{TRUE} will clean intermediate files that are created
+#' during rendering.
+#' @param params A list of named parameters that override custom params
+#' specified within the YAML front-matter (e.g. specifying a dataset to read or
+#' a date range to confine output to). Pass \code{"ask"} to start an
+#' application that helps guide parameter configuration.
+#' @param knit_meta (This option is reserved for expert use.) Metadata
+#' generated by \pkg{knitr}.
+#' @param envir The environment in which the code chunks are to be evaluated
+#' during knitting (can use \code{\link{new.env}()} to guarantee an empty new
+#' environment).
+#' @param run_pandoc An option for whether to run pandoc to convert Markdown
+#' output.
+#' @param quiet An option to suppress printing of the pandoc command line.
+#' @param encoding The encoding of the input file. See \code{\link{file}} for
+#' more information.
+#' @return
+#'   When \code{run_pandoc = TRUE}, the compiled document is written into
+#'   the output file, and the path of the output file is returned. When
+#'   \code{run_pandoc = FALSE}, the path of the Markdown output file, with
+#'   attributes \code{knit_meta} (the \pkg{knitr} meta data collected from code
+#'   chunks) and \code{intermediates} (the intermediate files/directories
+#'   generated by \code{render()}).
+#' @examples
+#' \dontrun{
+#' library(rmarkdown)
+#'
+#' # Render the default (first) format defined in the file
+#' render("input.Rmd")
+#'
+#' # Render all formats defined in the file
+#' render("input.Rmd", "all")
+#'
+#' # Render a single format
+#' render("input.Rmd", "html_document")
+#'
+#' # Render multiple formats
+#' render("input.Rmd", c("html_document", "pdf_document"))
+#' }
 #' @export
 render <- function(input,
                    output_format = NULL,
@@ -150,7 +284,7 @@ render <- function(input,
   force(knit_root_dir)
 
   # execute within the input file's directory
-  oldwd <- setwd(dirname(tools::file_path_as_absolute(input)))
+  oldwd <- setwd(dirname(abs_path(input)))
   on.exit(setwd(oldwd), add = TRUE)
 
   # reset the name of the input file to be relative and calculate variations
@@ -185,7 +319,7 @@ render <- function(input,
       '---\n',
       'title: "', input, '"\n',
       'author: "', Sys.info()[["user"]], '"\n',
-      'date: "', date(), '"\n',
+      'date: "', Sys.Date(), '"\n',
       '---\n'
     , sep = "")
     if (!identical(encoding, "native.enc"))
@@ -194,7 +328,7 @@ render <- function(input,
   }
 
   # read the input file
-  input_lines <- read_lines_utf8(knit_input, encoding)
+  input_lines <- read_utf8(knit_input, encoding)
 
   # read the yaml front matter
   yaml_front_matter <- parse_yaml_front_matter(input_lines)
@@ -226,7 +360,10 @@ render <- function(input,
     # force various output options
     output_options$self_contained <- FALSE
     output_options$dependency_resolver <- function(deps) {
-      shiny_prerendered_dependencies <<- deps
+      shiny_prerendered_dependencies <<- list(
+        deps = deps,
+        packages = get_loaded_packages()
+      )
       list()
     }
   }
@@ -253,9 +390,16 @@ render <- function(input,
   }
   output_dir <- dirname(output_file)
 
+  # Stop the render process early if the output directory does not exist
+  if (!dir_exists(output_dir)) {
+    stop("The directory '", output_dir, "') does not not exist.",
+         call. = FALSE)
+  }
+
   # use output filename based files dir
   files_dir_slash <- file.path(output_dir, knitr_files_dir(basename(output_file)))
   files_dir <- pandoc_path_arg(files_dir_slash)
+  files_dir_fig <- list.files(files_dir, '^figure-.+')  # existing figure dir(s)
 
   # default to no cache_dir (may be generated by the knit)
   cache_dir <- NULL
@@ -341,11 +485,14 @@ render <- function(input,
     # default rendering and chunk options
     knitr::render_markdown()
     knitr::opts_chunk$set(tidy = FALSE, error = FALSE)
+    # the retina option does not make sense to non-HTML output formats
+    if (!grepl('[.]html$', output_file)) knitr::opts_chunk$set(fig.retina = NULL)
 
     # store info about the final output format in opts_knit
     knitr::opts_knit$set(
       rmarkdown.pandoc.from = output_format$pandoc$from,
       rmarkdown.pandoc.to = pandoc_to,
+      rmarkdown.pandoc.args = output_format$pandoc$args,
       rmarkdown.pandoc.id_prefix = id_prefix,
       rmarkdown.keep_md = output_format$keep_md,
       rmarkdown.df_print = output_format$df_print,
@@ -409,7 +556,7 @@ render <- function(input,
         if (identical(knitr::opts_current$get("label"), "global")) {
 
           # check list of previously evaludated global chunks
-          code_string <- paste(code, collapse = '\n')
+          code_string <- one_string(code)
           if (!code_string %in% .globals$evaluated_global_chunks) {
 
             # save it in our list of evaluated global chunks
@@ -462,11 +609,13 @@ render <- function(input,
       assign("params", params, envir = envir)
       lockBinding("params", envir)
       on.exit({
-        do.call("unlockBinding", list("params", envir))
-        if (hasParams)
-          assign("params", envirParams, envir = envir)
-        else
-          remove("params", envir = envir)
+        if (exists("params", envir = envir, inherits = FALSE)) {
+            do.call("unlockBinding", list("params", envir))
+          if (hasParams)
+            assign("params", envirParams, envir = envir)
+          else
+            remove("params", envir = envir)
+        }
       }, add = TRUE)
     }
 
@@ -548,13 +697,18 @@ render <- function(input,
   }
 
   # clean the files_dir if we've either been asking to clean supporting files or
-  # the knitr cache is active
-  if (output_format$clean_supporting && (is.null(cache_dir) || !dir_exists(cache_dir)))
-      intermediates <- c(intermediates, files_dir)
+  # the knitr cache is active; clean the figure-* dir instead of the whole
+  # files_dir if other subdirs are generated by another format and still needed:
+  # https://github.com/rstudio/rmarkdown/issues/1472 and also #1503
+  if (output_format$clean_supporting && !dir_exists(cache_dir)) intermediates <- c(
+    intermediates, if (identical(list.files(files_dir, '^figure-.+'), files_dir_fig)) {
+      files_dir
+    } else knitr::opts_chunk$get('fig.path')
+  )
 
   # read the input text as UTF-8 then write it back out
-  input_text <- read_lines_utf8(input, encoding)
-  writeLines(input_text, utf8_input, useBytes = TRUE)
+  input_text <- read_utf8(input, encoding)
+  write_utf8(input_text, utf8_input)
 
   if (run_pandoc) {
 
@@ -588,13 +742,16 @@ render <- function(input,
 
     convert <- function(output, citeproc = FALSE) {
 
-      # temporarily move figures to the intermediate dir if specified:
-      # https://github.com/rstudio/rmarkdown/issues/500
-      figures_dir <- gsub('/$', '', knitr::opts_chunk$get("fig.path"))
-      if (!is.null(intermediates_dir) && dir_exists(figures_dir)) {
-        figures_dir_tmp <- intermediates_loc(figures_dir)
-        move_dir(figures_dir, figures_dir_tmp)
-        on.exit(move_dir(figures_dir_tmp, figures_dir), add = TRUE)
+      # temporarily move figures and bib files to the intermediate dir if
+      # specified: https://github.com/rstudio/rmarkdown/issues/500
+      if (!is.null(intermediates_dir)) {
+        figures_dir <- gsub('/$', '', knitr::opts_chunk$get("fig.path"))
+        files <- list.files(figures_dir, full.names = TRUE, recursive = TRUE)
+        # https://github.com/rstudio/rmarkdown/issues/1358
+        if (citeproc) files <- c(files, yaml_front_matter[['bibliography']])
+        for (f in files) {
+          intermediates <<- c(intermediates, copy_file_with_dir(f, intermediates_dir))
+        }
       }
 
       # ensure we expand paths (for Windows where leading `~/` does
@@ -712,7 +869,7 @@ render <- function(input,
 
   if (run_pandoc) {
     # return the full path to the output file
-    output_file <- tools::file_path_as_absolute(output_file)
+    output_file <- abs_path(output_file)
     # attach the metadata specified as rmd_output_metadata in YAML
     if (length(output_meta <- output_metadata$get()))
       attr(output_file, 'rmd_output_metadata') <- output_meta
@@ -727,16 +884,15 @@ render <- function(input,
 
 #' Render supporting files for an input document
 #'
-#' Render (copy) required supporting files for an input document to the _files
-#' directory associated with the document.
-#'
-#' @param from Directory to copy from
-#' @param files_dir Directory to copy files into
-#' @param rename_to Optional rename of source directory after it is copied
+#' Render (copy) required supporting files for an input document to the
+#' \code{_files} directory that is associated with the document.
 #'
+#' @param from The directory from which the files should be copied.
+#' @param files_dir The directory that will receive the copied files.
+#' @param rename_to An option to rename the source directory after the copy
+#' operation is complete.
 #' @return The relative path to the supporting files. This path is suitable
 #' for inclusion in HTML\code{href} and \code{src} attributes.
-#'
 #' @export
 render_supporting_files <- function(from, files_dir, rename_to = NULL) {
 
