@@ -2,19 +2,7 @@
 #'
 #' Format for converting from R Markdown to an MS Word document.
 #'
-#' @inheritParams pdf_document
-#' @inheritParams html_document
-#'
-#' @param reference_docx Use the specified file as a style reference in
-#'   producing a docx file. For best results, the reference docx should be a
-#'   modified version of a docx file produced using pandoc. Pass "default"
-#'   to use the rmarkdown default styles.
-#'
-#' @return R Markdown output format to pass to \code{\link{render}}
-#'
-#' @details
-#'
-#' See the \href{http://rmarkdown.rstudio.com/word_document_format.html}{online
+#' See the \href{https://rmarkdown.rstudio.com/word_document_format.html}{online
 #' documentation} for additional details on using the \code{word_document} format.
 #'
 #' R Markdown documents can have optional metadata that is used to generate a
@@ -23,12 +11,17 @@
 #'
 #' R Markdown documents also support citations. You can find more information on
 #' the markdown syntax for citations in the
-#' \href{http://rmarkdown.rstudio.com/authoring_bibliographies_and_citations.html}{Bibliographies
+#' \href{https://rmarkdown.rstudio.com/authoring_bibliographies_and_citations.html}{Bibliographies
 #' and Citations} article in the online documentation.
-#'
+#' @inheritParams pdf_document
+#' @inheritParams html_document
+#' @param reference_docx Use the specified file as a style reference in
+#'   producing a docx file. For best results, the reference docx should be a
+#'   modified version of a docx file produced using pandoc. Pass "default"
+#'   to use the rmarkdown default styles.
+#' @return R Markdown output format to pass to \code{\link{render}}
 #' @examples
 #' \dontrun{
-#'
 #' library(rmarkdown)
 #'
 #' # simple invocation
@@ -37,7 +30,6 @@
 #' # specify an option for syntax highlighting
 #' render("input.Rmd", word_document(highlight = "zenburn"))
 #' }
-#'
 #' @export
 word_document <- function(toc = FALSE,
                           toc_depth = 3,
@@ -82,13 +74,21 @@ word_document <- function(toc = FALSE,
   args <- c(args, pandoc_highlight_args(highlight))
 
   # reference docx
-  if (!is.null(reference_docx) && !identical(reference_docx, "default")) {
-    args <- c(args, reference_doc_arg("docx"), pandoc_path_arg(reference_docx))
-  }
+  args <- c(args, reference_doc_args("docx", reference_docx))
 
   # pandoc args
   args <- c(args, pandoc_args)
 
+  saved_files_dir <- NULL
+  pre_processor <- function(metadata, input_file, runtime, knit_meta, files_dir, output_dir) {
+    saved_files_dir <<- files_dir
+    NULL
+  }
+
+  intermediates_generator <- function(...) {
+    reference_intermediates_generator(saved_files_dir, ..., reference_docx)
+  }
+
   # return output format
   output_format(
     knitr = knitr,
@@ -96,13 +96,16 @@ word_document <- function(toc = FALSE,
                             from = from_rmarkdown(fig_caption, md_extensions),
                             args = args),
     keep_md = keep_md,
-    df_print = df_print
+    df_print = df_print,
+    pre_processor = pre_processor,
+    intermediates_generator = intermediates_generator
   )
 }
 
-reference_doc_arg <- function(type) {
-  paste0("--reference-", if (pandoc2.0()) "doc" else {
-    match.arg(type, c("docx", "odt"))
-  })
+reference_doc_args <- function(type, doc) {
+  if (is.null(doc) || identical(doc, "default")) return()
+  c(paste0("--reference-", if (pandoc2.0()) "doc" else {
+    match.arg(type, c("docx", "odt", "doc"))
+  }), pandoc_path_arg(doc))
 }
 
